from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import User


class AssumableModelBackend(ModelBackend):
    """
    Custom authentication backend that allows authentication without a
    password.

    IMPORTANT: This backend assumes that the credentials have already been
    verified if a password is not passed to authenticate(), so all calls to
    that method should be done very carefully.
    """

    def authenticate(self, username=None, password=None):
        try:
            user = User.objects.get(username=username)
            # If no password was supplied, assume that we've already verified credentials
            if password == None:
                return user
            # Otherwise, check the password
            elif user.check_password(password):
                return user
        except User.DoesNotExist:
            return None


class AssumableEmailOrUsernameModelBackend(ModelBackend):
    """
    Custom authentication backend that allows logging in using either username
    or email address, and allows authentication without a password.

    IMPORTANT: This backend assumes that the credentials have already been
    verified if a password is not passed to authenticate(), so all calls to
    that method should be done very carefully.
    """

    def authenticate(self, username=None, password=None):
        if '@' in username:
            kwargs = {'email': username}
        else:
            kwargs = {'username': username}
        try:
            user = User.objects.get(**kwargs)
            # If no password was supplied, assume that we've already verified credentials
            if password == None:
                return user
            # Otherwise, check the password
            elif user.check_password(password):
                return user
        except User.DoesNotExist:
            return None



from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.conf import settings
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth import authenticate, SESSION_KEY, BACKEND_SESSION_KEY
from django.contrib.auth.models import User
from django.contrib import messages


CAN_ASSUME_STAFF = getattr(settings, 'CAN_ASSUME_STAFF', False)
URL_AFTER_ASSUME = getattr(settings, 'URL_AFTER_ASSUME', '/')


@staff_member_required
def assume_user(request, id, next_url=URL_AFTER_ASSUME):
    """
    Custom admin view that logs into a specified user account.
    """

    user = get_object_or_404(User, pk=id)

    # Check whether staff members can be assumed
    if user.is_staff and not CAN_ASSUME_STAFF:
        messages.error(request, "Sorry, staff members cannot be assumed.")
        return HttpResponseRedirect(reverse('admin:auth_user_change', args=(user.id,)))

    # Call authenticate without a password (this only works with our custom backend)
    user = authenticate(username=user.username)

    # We could just call django.contrib.auth.login(request, user) here, but
    # instead we use duplicate code so that the last_login field doesn't get
    # updated when a user is assumed.
    if SESSION_KEY in request.session:
        if request.session[SESSION_KEY] != user.id:
            # To avoid reusing another user's session, create a new, empty
            # session if the existing session corresponds to a different
            # authenticated user.
            request.session.flush()
    else:
        request.session.cycle_key()
    request.session[SESSION_KEY] = user.id
    request.session[BACKEND_SESSION_KEY] = user.backend
    if hasattr(request, 'user'):
        request.user = user
    # End of custom login

    return HttpResponseRedirect(next_url)



import os
from setuptools import setup, find_packages

version = '0.1'

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='django-assume',
    version = version,
    description = "Django app that allows administrators to log in to user " \
                  "accounts without having to provide a password",
    long_description = read('README.rst'),
    classifiers = [],
    keywords = "",
    author = "Bryan Chow",
    author_email = '',
    url = 'https://github.com/bryanchow/django-assume',
    download_url = 'https://github.com/bryanchow/django-assume/tarball/master',
    license = "WTFPL",
    packages = find_packages(exclude=['ez_setup', 'examples', 'tests']),
    include_package_data = True,
    zip_safe = False,
    install_requires = [
        'django',
    ],
)
